@@ -1,5 +1,5 @@
 <?php
-/* SVN FILE: $Id: datasource.php 5612 2007-08-30 01:49:55Z phpnut $ */
+/* SVN FILE: $Id: datasource.php 8166 2009-05-04 21:17:19Z gwoo $ */
 /**
  * DataSource base class
  *
@@ -7,32 +7,30 @@
  *
  * PHP versions 4 and 5
  *
- * CakePHP(tm) :  Rapid Development Framework <http://www.cakephp.org/>
- * Copyright 2005-2007, Cake Software Foundation, Inc.
- *								1785 E. Sahara Avenue, Suite 490-204
- *								Las Vegas, Nevada 89104
+ * CakePHP(tm) :  Rapid Development Framework (http://www.cakephp.org)
+ * Copyright 2005-2008, Cake Software Foundation, Inc. (http://www.cakefoundation.org)
  *
  * Licensed under The MIT License
  * Redistributions of files must retain the above copyright notice.
  *
  * @filesource
- * @copyright		Copyright 2005-2007, Cake Software Foundation, Inc.
- * @link				http://www.cakefoundation.org/projects/info/cakephp CakePHP(tm) Project
- * @package			cake
- * @subpackage		cake.cake.libs.model.datasources
- * @since			CakePHP(tm) v 0.10.5.1790
- * @version			$Revision: 5612 $
- * @modifiedby		$LastChangedBy: phpnut $
- * @lastmodified	$Date: 2007-08-29 20:49:55 -0500 (Wed, 29 Aug 2007) $
- * @license			http://www.opensource.org/licenses/mit-license.php The MIT License
+ * @copyright     Copyright 2005-2008, Cake Software Foundation, Inc. (http://www.cakefoundation.org)
+ * @link          http://www.cakefoundation.org/projects/info/cakephp CakePHP(tm) Project
+ * @package       cake
+ * @subpackage    cake.cake.libs.model.datasources
+ * @since         CakePHP(tm) v 0.10.5.1790
+ * @version       $Revision: 8166 $
+ * @modifiedby    $LastChangedBy: gwoo $
+ * @lastmodified  $Date: 2009-05-04 14:17:19 -0700 (Mon, 04 May 2009) $
+ * @license       http://www.opensource.org/licenses/mit-license.php The MIT License
  */
 /**
  * DataSource base class
  *
  * Long description for file
  *
- * @package		cake
- * @subpackage	cake.cake.libs.model.datasources
+ * @package       cake
+ * @subpackage    cake.cake.libs.model.datasources
  */
 class DataSource extends Object {
 /**
@@ -42,13 +40,6 @@ class DataSource extends Object {
  * @access public
  */
 	var $connected = false;
-/**
- * Print debug info?
- *
- * @var boolean
- * @access public
- */
-	var $debug = false;
 /**
  * Print full query debug info?
  *
@@ -84,6 +75,18 @@ class DataSource extends Object {
  * @access public
  */
 	var $took = null;
+/**
+ * The starting character that this DataSource uses for quoted identifiers.
+ *
+ * @var string
+ */
+	var $startQuote = null;
+/**
+ * The ending character that this DataSource uses for quoted identifiers.
+ *
+ * @var string
+ */
+	var $endQuote = null;
 /**
  * Enter description here...
  *
@@ -178,19 +181,18 @@ class DataSource extends Object {
  */
 	var $_transactionStarted = false;
 /**
- * Enter description here...
+ * Whether or not source data like available tables and schema descriptions
+ * should be cached
  *
  * @var boolean
  */
-       var $cacheSources = true;
+	var $cacheSources = true;
 /**
  * Constructor.
  */
-	function __construct() {
+	function __construct($config = array()) {
 		parent::__construct();
-		if (func_num_args() > 0) {
-			$this->setConfig(func_get_arg(0));
-		}
+		$this->setConfig($config);
 	}
 /**
  * Caches/returns cached results for child instances
@@ -201,36 +203,33 @@ class DataSource extends Object {
 		if ($this->cacheSources === false) {
 			return null;
 		}
-		if ($this->_sources != null) {
+
+		if ($this->_sources !== null) {
 			return $this->_sources;
 		}
 
-		if (Configure::read() > 0) {
-			$expires = "+30 seconds";
-		} else {
-			$expires = "+999 days";
+		$key = ConnectionManager::getSourceName($this) . '_' . $this->config['database'] . '_list';
+		$key = preg_replace('/[^A-Za-z0-9_\-.+]/', '_', $key);
+		$sources = Cache::read($key, '_cake_model_');
+
+		if (empty($sources)) {
+			$sources = $data;
+			Cache::write($key, $data, '_cake_model_');
 		}
 
-		if ($data != null) {
-			$data = serialize($data);
-		}
-		$filename = ConnectionManager::getSourceName($this) . '_' . preg_replace("/[^A-Za-z0-9_-]/", "_", $this->config['database']) . '_list';
-		$new = cache('models' . DS . $filename, $data, $expires);
-
-		if ($new != null) {
-			$new = unserialize($new);
-			$this->_sources = $new;
-		}
-		return $new;
+		$this->_sources = $sources;
+		return $sources;
 	}
 /**
  * Convenience method for DboSource::listSources().  Returns source names in lowercase.
  *
  * @return array
  */
-	function sources() {
-		$return = array_map('strtolower', $this->listSources());
-		return $return;
+	function sources($reset = false) {
+		if ($reset === true) {
+			$this->_sources = null;
+		}
+		return array_map('strtolower', $this->listSources());
 	}
 /**
  * Returns a Model description (metadata) or null if none found.
@@ -242,17 +241,42 @@ class DataSource extends Object {
 		if ($this->cacheSources === false) {
 			return null;
 		}
-		if (isset($this->__descriptions[$model->tablePrefix . $model->table])) {
-			return $this->__descriptions[$model->tablePrefix . $model->table];
+		$table = $this->fullTableName($model, false);
+		if (isset($this->__descriptions[$table])) {
+			return $this->__descriptions[$table];
 		}
-		$cache = $this->__cacheDescription($model->tablePrefix . $model->table);
+		$cache = $this->__cacheDescription($table);
 
 		if ($cache !== null) {
-			$this->__descriptions[$model->tablePrefix . $model->table] =& $cache;
+			$this->__descriptions[$table] =& $cache;
 			return $cache;
 		}
 		return null;
 	}
+/**
+ * Begin a transaction
+ *
+ * @return boolean Returns true if a transaction is not in progress
+ */
+	function begin(&$model) {
+		return !$this->_transactionStarted;
+	}
+/**
+ * Commit a transaction
+ *
+ * @return boolean Returns true if a transaction is in progress
+ */
+	function commit(&$model) {
+		return $this->_transactionStarted;
+	}
+/**
+ * Rollback a transaction
+ *
+ * @return boolean Returns true if a transaction is in progress
+ */
+	function rollback(&$model) {
+		return $this->_transactionStarted;
+	}
 /**
  * Converts column types to basic types
  *
@@ -351,43 +375,33 @@ class DataSource extends Object {
  * @param array $config The configuration array
  * @return void
  */
-	function setConfig($config) {
-		if (is_array($this->_baseConfig)) {
-			$this->config = $this->_baseConfig;
-			foreach ($config as $key => $val) {
-				$this->config[$key] = $val;
-			}
-		}
+	function setConfig($config = array()) {
+		$this->config = array_merge($this->_baseConfig, $this->config, $config);
 	}
 /**
  * Cache the DataSource description
  *
  * @param string $object The name of the object (model) to cache
  * @param mixed $data The description of the model, usually a string or array
- * @return void
  */
 	function __cacheDescription($object, $data = null) {
 		if ($this->cacheSources === false) {
 			return null;
 		}
-		if (Configure::read() > 0) {
-			$expires = "+15 seconds";
-		} else {
-			$expires = "+999 days";
-		}
 
 		if ($data !== null) {
 			$this->__descriptions[$object] =& $data;
-			$cache = serialize($data);
-		} else {
-			$cache = null;
 		}
-		$new = cache('models' . DS . ConnectionManager::getSourceName($this) . '_' . $object, $cache, $expires);
 
-		if ($new != null) {
-			$new = unserialize($new);
+		$key = ConnectionManager::getSourceName($this) . '_' . $object;
+		$cache = Cache::read($key, '_cake_model_');
+
+		if (empty($cache)) {
+			$cache = $data;
+			Cache::write($key, $cache, '_cake_model_');
 		}
-		return $new;
+
+		return $cache;
 	}
 /**
  * Enter description here...
@@ -408,11 +422,11 @@ class DataSource extends Object {
 			$val = null;
 
 			if (strpos($query, $key) !== false) {
-				switch($key) {
+				switch ($key) {
 					case '{$__cakeID__$}':
-						if (isset($data[$model->name]) || isset($data[$association])) {
-							if (isset($data[$model->name][$model->primaryKey])) {
-								$val = $data[$model->name][$model->primaryKey];
+						if (isset($data[$model->alias]) || isset($data[$association])) {
+							if (isset($data[$model->alias][$model->primaryKey])) {
+								$val = $data[$model->alias][$model->primaryKey];
 							} elseif (isset($data[$association][$model->primaryKey])) {
 								$val = $data[$association][$model->primaryKey];
 							}
@@ -437,8 +451,8 @@ class DataSource extends Object {
 									if (isset($assoc['foreignKey'])) {
 										$foreignKey = $assoc['foreignKey'];
 
-										if (isset($data[$model->name][$foreignKey])) {
-											$val = $data[$model->name][$foreignKey];
+										if (isset($data[$model->alias][$foreignKey])) {
+											$val = $data[$model->alias][$foreignKey];
 										} elseif (isset($data[$association][$foreignKey])) {
 											$val = $data[$association][$foreignKey];
 										} else {
@@ -464,7 +478,7 @@ class DataSource extends Object {
 				if (empty($val) && $val !== '0') {
 					return false;
 				}
-				$query = r($key, $this->value($val, $model->getColumnType($model->primaryKey)), $query);
+				$query = str_replace($key, $this->value($val, $model->getColumnType($model->primaryKey)), $query);
 			}
 		}
 		return $query;
@@ -477,13 +491,17 @@ class DataSource extends Object {
  * @return unknown
  */
 	function resolveKey($model, $key) {
-		return $model->name . $key;
+		return $model->alias . $key;
 	}
 /**
  * Closes the current datasource.
  *
  */
 	function __destruct() {
+		if ($this->_transactionStarted) {
+			$null = null;
+			$this->rollback($null);
+		}
 		if ($this->connected) {
 			$this->close();
 		}
